' MOUSE.BAS, v1.2
'
' Copyright (C) 2002 Michael Webster
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License as
' published by the Free Software Foundation; either version 2 of
' the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public
' License along with this program; if not, write to the Free
' Software Foundation, Inc., 59 Temple Place, Suite 330,
' Boston, MA  02111-1307  USA

DECLARE SUB CheckMouse ()
DEFINT A-Z

' This file contains a QuickBASIC module consisting of a minimal set of
' procedures for adding mouse support to a QuickBASIC program.
'
' To use these procedures within the QuickBASIC or QuickBASIC Extended
' programming environment, you must load the default Quick library
' (QB.QLB or QBX.QLB) when the programming environment is started.
' Refer to QUICKLIB.TXT for more information.
'
' The CheckMouse procedure and the shared flags sfMouseOK and sfMouseChecked
' are private to this module. Other modules can determine if a mouse driver
' is present by calling the GetMouseStatus procedure and checking the value
' returned in the row argument (a non-zero value indicates that a mouse
' driver is present).
'
' Calls to the procedures in this module will fail silently, and harmlessly,
' if a mouse driver is not present.
'
' The mouse driver works with the mouse cursor position in terms of a
' 640x200 virtual screen with the upper left corner as row 0, col 0.
' The procedures in this module work with the mouse cursor position in
' terms of an 80x25 text mode screen with the upper left corner as
' row 1, col 1.
'
' Note that calling any of the procedures in this module will force the
' screen mode to 80x25 text.
'
' The mouse cursor should be hidden during any operation that reads from,
' writes to, or changes the number of, the visible screen page. If the
' mouse cursor is not hidden before reading data from the visible screen
' page, then the data read from the mouse cursor location will be the data
' for the mouse cursor. As part of the process of moving the mouse cursor
' on the screen, the mouse driver saves and then restores the screen data
' at the mouse cursor location. The mouse driver hides the mouse cursor
' by restoring the saved screen data. If the mouse cursor is not hidden
' before changing the screen data at the mouse cursor location (with a
' write operation or by changing the visible screen page), then the mouse
' driver will restore "old" data when the mouse cursor is moved.
'
' ===================================================================
' Changes in v1.1:
'
'   Added the SetCursorMasks procedure.
'
' Changes in v1.2:
'
'   Added error logging feature to parameter check code.
'
' ===================================================================

' $INCLUDE: 'INTX.BI'
' $INCLUDE: 'ERROR.BI'
' $INCLUDE: 'GENERAL.BI'
' $INCLUDE: 'MOUSE.BI'

CONST cFalse = 0, cTrue = NOT cFalse

' Declare the shared flags.
DIM SHARED sfMouseChecked, sfMouseOK

' *** START TEST CODE ***
' *** END TEST CODE ***

SUB CheckMouse

    ' Forces the screen mode to 80x25 text, checks for a mouse driver,
    ' and, if a driver is present, calls the mouse driver Mouse Reset
    ' and Status function. Sets the shared flag sfMouseOK if the mouse
    ' driver is present and the reset succeeded. Sets the shared flag
    ' sfMouseChecked on the first call so the other procedures in this
    ' module can avoid unnecessary calls to this procedure.

    DIM regX AS RegTypeX

    SCREEN 0
    WIDTH 80

    sfMouseChecked = cTrue

    ' Programs typically communicate with the mouse driver by calling
    ' Interrupt 33h. When the CPU executes an Interrupt instruction,
    ' it reads the interrupt vector (the segment and offset addresses
    ' of the interrupt handler) from a table at segment 0 in memory.
    ' Interrupt vectors are 4 bytes each with the vector for Interrupt
    ' 0 at offset 0 in the table, so the offset of an interrupt vector
    ' (in segment 0) is the interrupt number * 4.
    '
    ' Get the the mouse driver interrupt vector and the byte that
    ' the vector points to.
    mouseVector& = MemDword(0, &H33 * 4)
    firstByte = MemByte(UInt(HighWord(mouseVector&)), UInt(LowWord(mouseVector&)))

    ' If the mouse interrupt vector is 0000:0000 (possible for very old
    ' versions of MS-DOS ?) or the vector points to an IRET instruction
    ' (op code CFh), then the mouse driver is not present.
    IF mouseVector& = 0 OR firstByte = &HCF THEN EXIT SUB

    regX.ax = 0
    InterruptX &H33, regX, regX

    ' The function will return with AX set to -1 if the
    ' reset succeeded, or to 0 if the reset failed.
    sfMouseOK = regX.ax

END SUB

SUB GetMouseStatus (row%, col%, fLeft%, fRight%) STATIC

    ' Calls the mouse driver Get Button Status and Mouse Position
    ' function. Returns with the arguments set to reflect the current
    ' mouse status. The <row%> and <col%> arguments are set to the
    ' current mouse cursor coordinates. The <fLeft%> and <fRight%>
    ' arguments are set to true if the corresponding button is pressed.
    ' All arguments are set to 0 if a mouse driver is not present.

    DIM regX AS RegTypeX

    IF NOT sfMouseChecked THEN CheckMouse

    IF sfMouseOK THEN
     	  regX.ax = 3
     	  InterruptX &H33, regX, regX
     	  row = regX.dx \ 8 + 1
     	  col = regX.cx \ 8 + 1
     	  IF regX.bx AND 1 THEN fLeft = cTrue ELSE fLeft = cFalse
     	  IF regX.bx AND 2 THEN fRight = cTrue ELSE fRight = cFalse
    ELSE
     	  row = 0
     	  col = 0
     	  left = 0
     	  right = 0
    END IF

END SUB

SUB HideMouseCursor

    ' Calls the mouse driver Hide Cursor function. This function causes
    ' the mouse driver to remove the mouse cursor from the screen and
    ' decrement its internal cursor flag.
    '
    ' Note that the mouse driver decrements the flag with each call even
    ' if the mouse cursor is hidden. If this procedure is called multiple
    ' times without intervening calls to ShowMouseCursor, then the mouse
    ' driver will not redisplay the cursor until ShowMouseCursor has been
    ' called an equal number of times. Refer to ShowMouseCursor for more
    ' information.

    DIM regX AS RegTypeX

    IF NOT sfMouseChecked THEN CheckMouse

    IF sfMouseOK THEN
     	  regX.ax = 2
     	  InterruptX &H33, regX, regX
    END IF

END SUB

SUB HideMouseCursorIf (row%, col%, rows%, cols%)

    ' Removes the mouse cursor from the screen if it is within the
    ' rectangular region specified by the arguments.
    '
    ' This procedure avoids the Conditional Off function provided by
    ' the mouse driver because that function may not work correctly
    ' with all mouse drivers and/or operating environments.

    lastRow = row + rows - 1
    lastCol = col + cols - 1

    IF row < 1 THEN
     	  LogError "MOUSE.BAS", "HideMouseCursorIf", "row < 1"
     	  ERROR cErrorIllegalFunctionCall
    END IF
    IF lastRow > 25 THEN
     	  LogError "MOUSE.BAS", "HideMouseCursorIf", "lastRow > 25"
     	  ERROR cErrorIllegalFunctionCall
    END IF
    IF col < 1 THEN
     	  LogError "MOUSE.BAS", "HideMouseCursorIf", "col < 1"
     	  ERROR cErrorIllegalFunctionCall
    END IF
    IF lastCol > 80 THEN
     	  LogError "MOUSE.BAS", "HideMouseCursorIf", "lastCol > 80"
     	  ERROR cErrorIllegalFunctionCall
    END IF

    GetMouseStatus mouseRow, mouseCol, 0, 0

    ' If the mouse cursor is within the specified region, then hide it.
    IF mouseRow >= row AND mouseRow <= lastRow AND mouseCol >= col AND mouseCol <= lastCol THEN
     	  HideMouseCursor
    END IF

END SUB

SUB SetCursorMasks (screenMask%, cursorMask%)

    ' Calls the mouse driver Set Text Cursor function to set the mouse
    ' driver's software text cursor masks to the values specified by
    ' <screenMask%> and <cursorMask%>.
    '
    ' The mouse driver uses the software text cursor masks to control
    ' the appearance of the text mode mouse cursor. The mouse driver
    ' creates the mouse cursor data by combining the screen data at
    ' the mouse cursor location with the masks as follows:
    '
    '   cursorData = (screenData AND screenMask) XOR cursorMask
    '
    ' The screen data contains a character code in the least significant
    ' byte and a character attribute in the most significant byte.
    '
    ' The mouse driver's default screen mask is 77FFh and the default
    ' cursor mask is 7700h. These values cause the mouse driver to
    ' clear the blink/intensity bits and invert the colors in the
    ' attribute byte, and to leave the character code byte unchanged.
    '
    ' To display a specific character in the mouse cursor, clear the
    ' least significant byte of the screen mask and load the character
    ' code into the least significant byte of the cursor mask.
    '
    ' Note that the mouse driver will revert back to the default masks
    ' whenever it detects a video mode set.

    DIM regX AS RegTypeX

    IF NOT sfMouseChecked THEN CheckMouse

    IF sfMouseOK THEN

     	  ' The cursor must be visible.
     	  ShowMouseCursor

     	  regX.ax = &HA
     	  ' Select the software text cursor.
     	  regX.bx = 0
     	  regX.cx = screenMask
     	  regX.dx = cursorMask
     	  InterruptX &H33, regX, regX

    END IF

END SUB

SUB SetMousePosition (row%, col%)

    ' Calls the mouse driver Set Mouse Cursor Position function to set
    ' the mouse cursor position to <row%>, <col%>.

    DIM regX AS RegTypeX

    IF NOT sfMouseChecked THEN CheckMouse

    IF row < 1 OR row > 25 THEN
     	  LogError "MOUSE.BAS", "SetMousePosition", "row < 1 OR row > 25"
     	  ERROR cErrorIllegalFunctionCall
    END IF
    IF col < 1 OR col > 80 THEN
     	  LogError "MOUSE.BAS", "SetMousePosition", "col < 1 OR col > 80"
     	  ERROR cErrorIllegalFunctionCall
    END IF

    IF sfMouseOK THEN
     	  regX.ax = 4
     	  regX.cx = (col - 1) * 8
     	  regX.dx = (row - 1) * 8
     	  InterruptX &H33, regX, regX
    END IF

END SUB

SUB ShowMouseCursor

    ' Calls the mouse driver Show Cursor function. This function causes
    ' the mouse driver to increment its internal cursor flag and, if the
    ' flag is 0 after being incremented, display the mouse cursor. The
    ' flag will never be greater than 0.

    DIM regX AS RegTypeX

    IF NOT sfMouseChecked THEN CheckMouse

    IF sfMouseOK THEN
     	  regX.ax = 1
     	  InterruptX &H33, regX, regX
    END IF

END SUB

